==> cubicweb-testcard-0.2.1.tar.gz: cubicweb-testcard-0.2.1/hooks.py <==
from itertools import chain

from cubicweb.selectors import implements
from cubicweb.server import hook

class MakeTestInstancesOnVersionStatusChange(hook.Hook):
    """when a Version is going from 'planned' to 'in development', search
    for related test cards and create a test instance for each of them
    """
    __regid__ = 'createtestinstances'
    __select__ = hook.Hook.__select__ & implements('TrInfo')
    events = ('after_add_entity',)

    def __call__(self):
        forentity = self.entity.for_entity
        if forentity.e_schema  != 'Version':
            return
        if self.entity.new_state.name == 'dev':
            self.copy_cards(forentity.eid)

    def copy_cards(self, veid):
        # get cards for past stories
        rset1 = self._cw.execute('Any X,XT WHERE X test_case_of P, X test_case_for S, S in_state ST,'
                                'ST name "done", V version_of P, V eid %(v)s, X title XT,'
                                'NOT EXISTS(TI instance_of X, TI for_version V)',
                               {'v': veid}, 'v')
        # get cards for stories in this version
        rset2 = self._cw.execute('Any X,XT WHERE X test_case_of P, X test_case_for S,'
                                'S done_in V, V eid %(v)s, X title XT,'
                                'NOT EXISTS(TI instance_of X, TI for_version V)',
                               {'v': veid}, 'v')
        # get general cards (not related to a particular story
        rset3 = self._cw.execute('Any X,XT WHERE X test_case_of P, NOT X test_case_for S,'
                                'V version_of P, V eid %(v)s, X title XT,'
                                'NOT EXISTS(TI instance_of X, TI for_version V)',
                               {'v': veid}, 'v')
        done = set()
        for eid, title in chain(rset1, rset2, rset3):
            if eid in done:
                continue
            done.add(eid)
            self._cw.unsafe_execute('INSERT TestInstance X: X name %(name)s,'
                                   'X instance_of C, X for_version V '
                                   'WHERE C eid %(c)s, V eid %(v)s',
                                   {'v': veid, 'c': eid, 'name': title})

==> cubicweb-testcard-0.2.2.tar.gz: cubicweb-testcard-0.2.2/hooks.py <==
from itertools import chain

from cubicweb.selectors import implements
from cubicweb.server import hook

class MakeTestInstancesOnVersionStatusChange(hook.Hook):
    """when a Version is going from 'planned' to 'in development', search
    for related test cards and create a test instance for each of them
    """
    __regid__ = 'createtestinstances'
    __select__ = hook.Hook.__select__ & implements('TrInfo')
    events = ('after_add_entity',)

    def __call__(self):
        forentity = self.entity.for_entity
        if forentity.e_schema  != 'Version':
            return
        if self.entity.new_state.name == 'dev':
            self.copy_cards(forentity.eid)

    def copy_cards(self, veid):
        # get cards for past stories
        rset1 = self._cw.execute('Any X,XT WHERE X test_case_of P, X test_case_for S, S in_state ST,'
                                'ST name "done", V version_of P, V eid %(v)s, X title XT,'
                                'NOT EXISTS(TI instance_of X, TI for_version V)',
                               {'v': veid})
        # get cards for stories in this version
        rset2 = self._cw.execute('Any X,XT WHERE X test_case_of P, X test_case_for S,'
                                'S done_in V, V eid %(v)s, X title XT,'
                                'NOT EXISTS(TI instance_of X, TI for_version V)',
                               {'v': veid})
        # get general cards (not related to a particular story
        rset3 = self._cw.execute('Any X,XT WHERE X test_case_of P, NOT X test_case_for S,'
                                'V version_of P, V eid %(v)s, X title XT,'
                                'NOT EXISTS(TI instance_of X, TI for_version V)',
                               {'v': veid})
        done = set()
        for eid, title in chain(rset1, rset2, rset3):
            if eid in done:
                continue
            done.add(eid)
            self._cw.execute('INSERT TestInstance X: X name %(name)s,'
                             'X instance_of C, X for_version V '
                             'WHERE C eid %(c)s, V eid %(v)s',
                             {'v': veid, 'c': eid, 'name': title})
